"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("@abraham/reflection");
var isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
var METADATA_KEY = 'rpc:methods';
var Executable = /** @class */ (function () {
    function Executable() {
        this.methods = Reflect.getMetadata(METADATA_KEY, Object.create(this));
    }
    Executable.prototype.execute = function (instance, call) {
        var cr = {};
        cr.id = call.id;
        if (!this.methods || !this.methods.includes(call.method)) {
            cr.ok = false;
        }
        else {
            cr.value = instance[call.method].apply(instance, call.params);
            cr.ok = true;
        }
        return cr;
    };
    return Executable;
}());
var WSRPC = /** @class */ (function (_super) {
    __extends(WSRPC, _super);
    function WSRPC() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.callbacks = new Map();
        return _this;
    }
    WSRPC.prototype.invoke = function (ws, method, params) {
        var _this = this;
        var id = Math.random().toString(36).substring(2) + Date.now().toString(36);
        var p = new Promise(function (resolve, reject) { return _this.callbacks.set(id, { resolve: resolve, reject: reject }); });
        ws.send(JSON.stringify({ method: method, params: params, id: id }));
        return p;
    };
    WSRPC.prototype.handle = function (msg, ws, instance) {
        var pmsg = JSON.parse(msg.toString());
        if (pmsg.method) {
            var req = pmsg;
            ws.send(JSON.stringify(this.execute(instance, req)));
        }
        else {
            var rall = pmsg;
            var p = this.callbacks.get(rall.id);
            if (rall.ok)
                p.resolve(rall.value);
            else
                p.reject();
            this.callbacks.delete(rall.id);
        }
    };
    return WSRPC;
}(Executable));
var Server = /** @class */ (function (_super) {
    __extends(Server, _super);
    function Server() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Server.prototype.listen = function (port) {
        var _this = this;
        this.wss = new isomorphic_ws_1.default.Server({ port: port });
        this.wss.on('connection', function (ws) {
            var instance = Object.create(_this);
            instance.client = ws;
            ws.onmessage = function (msg) { _this.handle(msg.data, ws, instance); };
        });
    };
    Server.prototype.call = function (method) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        return _super.prototype.invoke.call(this, this.client, method, params);
    };
    return Server;
}(WSRPC));
exports.Server = Server;
var Client = /** @class */ (function (_super) {
    __extends(Client, _super);
    function Client() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Client.prototype.connect = function (address) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.cws = new isomorphic_ws_1.default(address);
            _this.cws.onopen = function () {
                _this.cws.onmessage = function (msg) { _this.handle(msg.data, _this.cws, _this); };
                resolve();
            };
            _this.cws.onerror = function (err) {
                reject(err);
            };
        });
    };
    Client.prototype.call = function (method) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        return _super.prototype.invoke.call(this, this.cws, method, params);
    };
    return Client;
}(WSRPC));
exports.Client = Client;
function remote(target, key) {
    var methods = Reflect.getOwnMetadata(METADATA_KEY, target) || [];
    methods.push(key);
    Reflect.defineMetadata(METADATA_KEY, methods, target);
}
exports.remote = remote;
//# sourceMappingURL=index.js.map